@@ -15,10 +15,13 @@
 #' @param local If TRUE, the expression is evaluated such that
 #' all assignments are done to local temporary environment, otherwise
 #' the assignments are done in the calling environment.
-#' @param gc If TRUE, the garbage collector run after the future
-#' is resolved (in the process that evaluated the future).
+#' @param gc If TRUE, the garbage collector run (in the process that
+#' evaluated the future) after the value of the future is collected.
+#' \emph{Some types of futures ignore this argument.}
 #' @param earlySignal Specified whether conditions should be signaled
 #' as soon as possible or not.
+#' @param label An optional character string label attached to the
+#' future.
 #' @param \dots Additional named elements of the future.
 #'
 #' @return An object of class \code{Future}.
@@ -29,14 +32,14 @@
 #' @seealso
 #' One function that creates a Future is \code{\link{future}()}.
 #' It returns a Future that evaluates an R expression in the future.
-#' An alternative approach is to use the \code{\link{\%<=\%}} infix
+#' An alternative approach is to use the \code{\link{\%<-\%}} infix
 #' assignment operator, which creates a future from the
 #' right-hand-side (RHS) R expression and assigns its future value
 #' to a variable as a \emph{\link[base]{promise}}.
 #'
 #' @export
 #' @name Future-class
-Future <- function(expr=NULL, envir=parent.frame(), substitute=FALSE, local=TRUE, gc=FALSE, earlySignal=FALSE, ...) {
+Future <- function(expr=NULL, envir=parent.frame(), substitute=FALSE, local=TRUE, gc=FALSE, earlySignal=FALSE, label=NULL, ...) {
   if (substitute) expr <- substitute(expr)
   args <- list(...)
 
@@ -47,6 +50,7 @@ Future <- function(expr=NULL, envir=parent.frame(), substitute=FALSE, local=TRUE
   core$local <- local
   core$gc <- gc
   core$earlySignal <- earlySignal
+  core$label <- label
 
   ## The current state of the future, e.g.
   ## 'created', 'running', 'finished', 'failed', 'interrupted'.
@@ -64,6 +68,9 @@ Future <- function(expr=NULL, envir=parent.frame(), substitute=FALSE, local=TRUE
 print.Future <- function(x, ...) {
   class <- class(x)
   cat(sprintf("%s:\n", class[1]))
+  label <- x$label
+  if (is.null(label)) label <- "<none>"
+  cat("Label: ", sQuote(label), "\n", sep="")
   cat("Expression:\n")
   print(x$expr)
   cat(sprintf("Local evaluation: %s\n", x$local))
@@ -127,6 +134,35 @@ assertOwner <- function(future, ...) {
 }
 
 
+#' Run a future
+#'
+#' @param future A \link{Future}.
+#' @param \dots Not used.
+#'
+#' @return The \link{Future} object.
+#'
+#' @details
+#' This function can only be called once per future.
+#' Further calls will result in an informative error.
+#' If a future is not run when its value is queried,
+#' then it is run at that point.
+#'
+#' @aliases run
+#' @rdname run
+#' @export
+#' @export run
+run.Future <- function(future, ...) {
+  if (future$state != 'created') {
+    stop("A future can only be launched once.")
+  }
+  
+  future
+}
+
+run <- function(...) UseMethod("run")
+
+
+
 #' The value of a future
 #'
 #' Gets the value of a future.  If the future is unresolved, then
@@ -148,6 +184,10 @@ assertOwner <- function(future, ...) {
 #' @export
 #' @export value
 value.Future <- function(future, signal=TRUE, ...) {
+  if (future$state == 'created') {
+    future <- run(future)
+  }
+
   if (!future$state %in% c('finished', 'failed', 'interrupted')) {
     msg <- sprintf("Internal error: value() called on a non-finished future: %s", class(future)[1])
     mdebug(msg)
@@ -182,7 +222,7 @@ resolved.Future <- function(x, ...) {
 #' Inject code for the next type of future to use for nested futures
 #'
 #' @param future Current future.
-#' @param ... Not used.
+#' @param \dots Not used.
 #'
 #' @return A future expression with code injected to set what
 #' type of future to use for nested futures, iff any.
@@ -218,21 +258,11 @@ getExpression <- function(future, ...) UseMethod("getExpression")
 
 #' @export
 getExpression.Future <- function(future, mc.cores=NULL, ...) {
-  strategies <- plan("list")
-
-  ## If end of future stack, fall back to using single-core
-  ## processing.  In this case we don't have to rely
-  ## on the future package.  Instead, we can use the
-  ## light-weight approach where we force the number of
-  ## cores available to be one.  This we achieve by
-  ## setting the number of _additional_ cores to be
-  ## zero (sic!).
-  if (length(strategies) == 0) {
-    mc.cores <- 0L
-  }
+##  mdebug("getExpression() ...")
 
   ## Should 'mc.cores' be set?
   if (!is.null(mc.cores)) {
+##    mdebug("getExpression(): setting mc.cores=%d inside future", mc.cores)
     ## FIXME: How can we guarantee that '...future.mc.cores.old'
     ## is not overwritten?  /HB 2016-03-14
     enter <- bquote({
@@ -249,64 +279,80 @@ getExpression.Future <- function(future, mc.cores=NULL, ...) {
     enter <- exit <- NULL
   }
 
-  if (length(strategies) > 0) {
-    exit <- bquote({
-      ## covr: skip=2
-      .(exit)
-      future::plan(.(strategies))
-    })
-  }
-
-  ## Identify package namespaces for strategies
-  pkgs <- lapply(strategies, FUN=environment)
-  pkgs <- lapply(pkgs, FUN=environmentName)
-  pkgs <- unique(unlist(pkgs))
-  pkgs <- intersect(pkgs, loadedNamespaces())
-  mdebug("Packages to be loaded by expression (n=%d): %s", length(pkgs), paste(sQuote(pkgs), collapse=", "))
-  
-  if (length(pkgs) > 0L) {
-    ## Sanity check by verifying packages can be loaded already here
-    ## If there is somethings wrong in 'pkgs', we get the error
-    ## already before launching the future.
-    for (pkg in pkgs) loadNamespace(pkg)
+  ## Reset future strategies upon exit of future
+  strategies <- plan("list")
+  stopifnot(length(strategies) >= 1L)
+  exit <- bquote({
+    ## covr: skip=2
+    .(exit)
+    future::plan(.(strategies))
+  })
 
+  ## Pass down the default or the remain set of future strategies?
+  strategiesR <- strategies[-1]
+##  mdebug("Number of remaining strategies: %d\n", length(strategiesR))
+  if (length(strategiesR) == 0L) {
+##    mdebug("Set plan('default') inside future")
+    ## Use default future strategy
     enter <- bquote({
-      ## covr: skip=3
-      .(enter)      
-      ## TROUBLESHOOTING: If the package fails to load, then library()
-      ## suppress that error and generates a generic much less
-      ## informative error message.  Because of this, we load the
-      ## namespace first (to get a better error message) and then
-      ## calls library(), which attaches the package. /HB 2016-06-16
-      ## NOTE: We use local() here such that 'pkg' is not assigned
-      ##       to the future environment. /HB 2016-07-03
-      local({
-        for (pkg in .(pkgs)) {
-          loadNamespace(pkg)
-          library(pkg, character.only=TRUE)
-        }
-      })
+      ## covr: skip=2
+      .(enter)
+      future::plan("default")
     })
   } else {
+##    mdebug("Set plan(<remaining strategies>) inside future")
+    ## Identify package namespaces for strategies
+    pkgs <- lapply(strategiesR, FUN=environment)
+    pkgs <- lapply(pkgs, FUN=environmentName)
+    pkgs <- unique(unlist(pkgs, use.names=FALSE))
+    pkgs <- intersect(pkgs, loadedNamespaces())
+##    mdebug("Packages to be loaded by expression (n=%d): %s", length(pkgs), paste(sQuote(pkgs), collapse=", "))
+      
+    if (length(pkgs) > 0L) {
+      ## Sanity check by verifying packages can be loaded already here
+      ## If there is somethings wrong in 'pkgs', we get the error
+      ## already before launching the future.
+      for (pkg in pkgs) loadNamespace(pkg)
+  
+      enter <- bquote({
+        ## covr: skip=3
+        .(enter)      
+        ## TROUBLESHOOTING: If the package fails to load, then library()
+        ## suppress that error and generates a generic much less
+        ## informative error message.  Because of this, we load the
+        ## namespace first (to get a better error message) and then
+        ## calls library(), which attaches the package. /HB 2016-06-16
+        ## NOTE: We use local() here such that 'pkg' is not assigned
+        ##       to the future environment. /HB 2016-07-03
+        local({
+          for (pkg in .(pkgs)) {
+            loadNamespace(pkg)
+            library(pkg, character.only=TRUE)
+          }
+        })
+      })
+    }
+  
+    ## Pass down future strategies
     enter <- bquote({
       ## covr: skip=2
       .(enter)
+      future::plan(.(strategiesR))
     })
+  } ## if (length(strategiesR) > 0L)
+
+  expr <- makeExpression(expr=future$expr, local=future$local, enter=enter, exit=exit)
+  if (getOption("future.debug", FALSE)) {
+    print(expr)
   }
 
-  if (length(strategies) >= 2L) {
-    enter <- bquote({
-      ## covr: skip=2
-      .(enter)
-      future::plan(.(strategies[-1]))
-    })
-  }
-
-  makeExpression(expr=future$expr, local=future$local, gc=future$gc, enter=enter, exit=exit)
+##  mdebug("getExpression() ... DONE")
+  
+  expr
 } ## getExpression()
 
 
-makeExpression <- function(expr, local=TRUE, gc=FALSE, globals.onMissing=getOption("future.globals.onMissing", "error"), enter=NULL, exit=NULL) {
+makeExpression <- function(expr, local=TRUE, globals.onMissing=getOption("future.globals.onMissing", "error"), enter=NULL, exit=NULL) {
   ## Evaluate expression in a local() environment?
   if (local) {
     a <- NULL; rm(list="a")  ## To please R CMD check
@@ -335,29 +381,15 @@ makeExpression <- function(expr, local=TRUE, gc=FALSE, globals.onMissing=getOpti
   ## evaluation in a local is optional, cf. argument 'local'.
   ## If this was mandatory, we could.  Instead we use
   ## a tryCatch() statement. /HB 2016-03-14
-  if (gc) {
-    expr <- substitute({
-      ## covr: skip=8
-      enter
-      ...future.value <- tryCatch({
-        body
-      }, finally = {
-        exit
-      })
-      gc(verbose=FALSE, reset=FALSE)
-      ...future.value
-    }, env=list(enter=enter, body=expr, exit=exit, cleanup=cleanup))
-  } else {
-    expr <- substitute({
-      ## covr: skip=6
-      enter
-      tryCatch({
-        body
-      }, finally = {
-        exit
-      })
-    }, env=list(enter=enter, body=expr, exit=exit))
-  }
+  expr <- substitute({
+    ## covr: skip=6
+    enter
+    tryCatch({
+      body
+    }, finally = {
+      exit
+    })
+  }, env=list(enter=enter, body=expr, exit=exit))
 
   expr
 } ## makeExpression()
