@@ -4,21 +4,10 @@
 #' which means that its \emph{value is computed and resolved in
 #' parallel in another process}.
 #'
-#' @param expr An R \link[base]{expression}.
-#' @param envir The \link{environment} in which the evaluation
-#' is done and from which globals are obtained.
-#' @param substitute If TRUE, argument \code{expr} is
-#' \code{\link[base]{substitute}()}:ed, otherwise not.
-#' @param globals If TRUE, global objects are validated at the point
-#' in time when the future is created (always before it is resolved),
-#' that is, they identified and located.  If some globals fail to be
-#' located, an informative error is generated.
+#' @inheritParams future
+#' @inheritParams multiprocess
 #' @param workers The maximum number of multicore futures that can
 #' be active at the same time before blocking.
-#' @param gc If TRUE, the garbage collector run after the future
-#' is resolved (in the process that evaluated the future).
-#' @param earlySignal Specified whether conditions should be signaled as soon as possible or not.
-#' @param \dots Not used.
 #'
 #' @return A \link{MulticoreFuture}
 #' If \code{workers == 1}, then all processing using done in the
@@ -46,7 +35,7 @@
 #' this function directly, but to register it via
 #' \code{\link{plan}(multicore)} such that it becomes the default
 #' mechanism for all futures.  After this \code{\link{future}()}
-#' and \code{\link{\%<=\%}} will create \emph{multicore futures}.
+#' and \code{\link{\%<-\%}} will create \emph{multicore futures}.
 #'
 #' @seealso
 #' For processing in multiple background R sessions, see
@@ -61,7 +50,7 @@
 #' system.
 #'
 #' @export
-multicore <- function(expr, envir=parent.frame(), substitute=TRUE, globals=TRUE, workers=availableCores(constraints="multicore"), gc=FALSE, earlySignal=FALSE, ...) {
+multicore <- function(expr, envir=parent.frame(), substitute=TRUE, globals=TRUE, workers=availableCores(constraints="multicore"), earlySignal=FALSE, label=NULL, ...) {
   ## BACKWARD COMPATIBILITY
   args <- list(...)
   if ("maxCores" %in% names(args)) {
@@ -70,7 +59,6 @@ multicore <- function(expr, envir=parent.frame(), substitute=TRUE, globals=TRUE,
   }
 
   if (substitute) expr <- substitute(expr)
-  globals <- as.logical(globals)
   workers <- as.integer(workers)
   stopifnot(is.finite(workers), workers >= 1L)
 
@@ -79,18 +67,13 @@ multicore <- function(expr, envir=parent.frame(), substitute=TRUE, globals=TRUE,
   ## Eager futures best reflect how multicore futures handle globals.
   if (workers == 1L || !supportsMulticore()) {
     ## covr: skip=1
-    return(eager(expr, envir=envir, substitute=FALSE, globals=globals, local=TRUE, gc=gc))
-  }
-
-  ## Validate globals at this point in time?
-  if (globals) {
-    exportGlobals(expr, envir=envir, target=NULL, tweak=tweakExpression, resolve=TRUE)
+    return(eager(expr, envir=envir, substitute=FALSE, globals=globals, local=TRUE, label=label))
   }
 
   oopts <- options(mc.cores=workers)
   on.exit(options(oopts))
 
-  future <- MulticoreFuture(expr=expr, envir=envir, substitute=FALSE, workers=workers, gc=gc, earlySignal=earlySignal)
+  future <- MulticoreFuture(expr=expr, envir=envir, substitute=FALSE, globals=globals, workers=workers, earlySignal=earlySignal, label=label)
   run(future)
 }
 class(multicore) <- c("multicore", "multiprocess", "future", "function")
