@@ -1,13 +1,8 @@
-library("future")
+source("incl/start.R")
 library("listenv")
 
-ovars <- ls()
-oopts <- options(warn=1L, mc.cores=2L, future.debug=TRUE)
-
 message("*** constant() ...")
 
-constant <- future:::constant
-
 ## No global variables
 f <- try(constant(42L), silent=FALSE)
 print(f)
@@ -32,8 +27,4 @@ stopifnot(y == 42L)
 
 message("*** constant() ... DONE")
 
-
-## Cleanup
-plan(eager)
-options(oopts)
-rm(list=setdiff(ls(), ovars))
+source("incl/end.R")
