@@ -41,3 +41,8 @@ whichIndex <- future:::whichIndex
 ## Local functions for test scripts
 printf <- function(...) cat(sprintf(...))
 mstr <- function(...) message(paste(capture.output(str(...)), collapse="\n"))
+attachLocally <- function(x, envir=parent.frame()) {
+  for (name in names(x)) {
+    assign(name, value=x[[name]], envir=envir)
+  }
+}
