@@ -1,9 +1,6 @@
-library("future")
+source("incl/start.R")
 library("listenv")
 
-ovars <- ls()
-oopts <- options(warn=1L, mc.cores=2L, future.debug=TRUE)
-
 message("*** transparent() ...")
 
 ## No global variables
@@ -44,8 +41,4 @@ print(f)
 
 message("*** transparent() ... DONE")
 
-
-## Cleanup
-plan(eager)
-options(oopts)
-rm(list=setdiff(ls(), ovars))
+source("incl/end.R")
